package com.objis.demohibernate.service;


import java.util.Iterator;
import java.util.List;

import org.hibernate.Criteria;
import org.hibernate.Hibernate;
import org.hibernate.Query;
import org.hibernate.Transaction;
import org.hibernate.classic.Session;
import org.hibernate.criterion.Criterion;
import org.hibernate.criterion.Order;
import org.hibernate.criterion.ProjectionList;
import org.hibernate.criterion.Projections;

import com.objis.demohibernate.domaine.Formateur;
import com.objis.demohibernate.domaine.Formation;
import com.objis.demohibernate.simple.util.HibernateUtil;


public class GestionnaireFormation {

	// Rendre persistant une formation 
	public Long creerFormation(Formation formation){

		Session session = HibernateUtil.getSessionFactory().openSession();
		Transaction tx = session.beginTransaction();

		Long formationId = null;
		try {
			session.saveOrUpdate(formation);
			tx.commit();
			formationId = formation.getId();
		} 
		catch (Exception e) {
			tx.rollback();
		} 
		finally{
			session.close();
		}

		return formationId;
	}

	@SuppressWarnings("unchecked")
	public List<Formateur> requeteHqlFormateurs(String hql, String nom) {
		System.out.println(" ---- requeteHqlFormateurs ---- hql : " + hql + " ---- nom : " + nom + " ---- ");
		List<Formateur> formateurs = null;

		Session session = HibernateUtil.getSessionFactory().openSession();
		Transaction tx = session.beginTransaction();

		Query query = session.createQuery(hql);
		if( nom != null && hql.indexOf('?') > 0 ) {
			query.setParameter(0, nom, Hibernate.STRING);
		}
		formateurs = query.list();

		for (Formateur formateur : formateurs) {
			System.out.println(">>> Nom du formateur : " + formateur.getNom());
		}

		tx.commit();
		session.close();

		return formateurs;
	}

	@SuppressWarnings("unchecked")
	public List<Formateur> requeteCriteriaFormateurs(Criterion criterion) {
		System.out.println(" ---- requeteCriteriaFormateurs ---- criterion : " + criterion + " ---- ");
		List<Formateur> formateurs = null;

		Session session = HibernateUtil.getSessionFactory().openSession();
		Transaction tx = session.beginTransaction();

		Criteria criteria = session.createCriteria(Formateur.class);
		if( criterion != null ) {
			criteria.add( criterion );
		} else {
			criteria.setFirstResult(1);
			criteria.setMaxResults(2);
			criteria.setFetchSize(1);
		}
		criteria.addOrder(Order.desc("nom"));
		formateurs = criteria.list();

		for (Formateur formateur : formateurs) {
			System.out.println(">>> Nom du formateur : " + formateur.getNom());
		}

		tx.commit();
		session.close();

		return formateurs;
	}

	public boolean requeteProjection() {
		System.out.println(" ---- requeteProjection ---- ");

		try{
			Session session = HibernateUtil.getSessionFactory().openSession();
			Criteria crit = session.createCriteria(Formation.class);
			ProjectionList proList = Projections.projectionList();
			proList.add(Projections.property("duree"));
			proList.add(Projections.property("theme"));
			crit.setProjection(proList);
			List<?> list = crit.list();
			Iterator<?> it = list.iterator();
			if(!it.hasNext()){
				System.out.println("pas de donnes");
			}
			else{
				while(it.hasNext()){
					Object[] row = (Object[])it.next();
					for(int i = 0; i < row.length;i++){
						System.out.print(" | ");
						System.out.print(row[i]);
					}
					System.out.println();
				}
			}
			session.close();
		}
		catch(Exception e){
			System.out.println(e.getMessage());
			return false;
		}

		return true;
	}

	public Formation getFormation(Long id){		
		Session session = HibernateUtil.getSessionFactory().openSession();
		// Rcup Formation (Chargement d'une formation  partir de son Id)
		Formation formation = (Formation) session.get(Formation.class,id);		
		session.close();		
		return formation;
	}
}
